# SPDX-License-Identifier: MIT


"""
A modern Pythonic implementation of Avro Phonetic.

Licensed under the terms of the MIT License.
"""

# Import local modules.
from .main import *

# Version information.
__version_info__ = (2024, 10, 12)
__version__ = ".".join(map(str, __version_info__))
